@@ -63,14 +63,14 @@ export BASEDIR=`pwd`
 # export CDNAME=debian
 
 # Building $codename cd set ...
-export CODENAME=buster
+export CODENAME=bookworm
 
 # By default use Debian installer packages from $CODENAME
 if [ -z "$DI_CODENAME" ]; then
 	export DI_CODENAME=$CODENAME
 fi
 # If you want backported d-i (e.g. by setting
-# DI_CODENAME=jessie-backports, then you'll almost definitely also
+# DI_CODENAME=bookworm-backports, then you'll almost definitely also
 # want to enable BACKPORTS below as well
 
 # Should we include some packages from backports? If so, point at a
@@ -86,8 +86,8 @@ fi
 # the Debian mirror.
 #export DI_WWW_HOME=default
 
-# Version number, "2.2 r0", "2.2 r1" etc.
-export DEBVERSION="10.0.0"
+# Version number, "10.11.0", "11.1.0", "testing", etc
+export DEBVERSION="12.0.0"
 
 # Official or non-official set.
 # NOTE: THE "OFFICIAL" DESIGNATION IS ONLY ALLOWED FOR IMAGES AVAILABLE
@@ -132,20 +132,23 @@ export OUT=/srv/mirror/debian-cd-test
 # This cannot reside on an NFS mount.
 export APTTMP=/srv/mirror/tmp/apt
 
-# Do I want to have NONFREE merged in the CD set
+# Do I want to have NONFREE merged in the CD set?
 # export NONFREE=1
 
-# Do I want to have CONTRIB merged in the CD set
+# Do I want to have CONTRIB merged in the CD set?
 export CONTRIB=1
 
-# Do I want to have NONFREE on a separate CD (the last CD of the CD set)
+# Where should I look for non-free packages?
+export NONFREE_COMPONENTS="non-free non-free-firmware"
+
+# Do I want to have NONFREE on separate image(s) at the end set?
 # WARNING: Don't use NONFREE and EXTRANONFREE at the same time !
 # export EXTRANONFREE=1
 
 # Do I want to force (potentially non-free) firmware packages to be
 # placed on disc 1? Will make installation much easier if systems
 # contain hardware that depends on this firmware
-# export FORCE_FIRMWARE=1
+export FORCE_FIRMWARE=1
 
 # If you have a $MIRROR/dists/$CODENAME/local/binary-$ARCH dir with 
 # local packages that you want to put on the CD set then
@@ -208,8 +211,8 @@ export JIGDO_CHECKSUM="md5"
 
 # Extra keys that you might want apt to trust. List their fingerprints
 # here and debian-cd will grab them from the user's keyring as needed
-# (The example here is the buster release key)
-#export ARCHIVE_EXTRA_KEYS="80D15823B7FD1561F9F7BCDDDC30D7C23CBBABEE"
+# (The example here is the bullseye release key)
+#export ARCHIVE_EXTRA_KEYS="1F89983E0081FDE018F3CC9673A4F27B8DD47936"
 
 # By default we use debootstrap --no-check-gpg to find out the minimal set
 # of packages because there's no reason to not trust the local mirror. But
@@ -439,6 +442,11 @@ if [ -d "/etc/ssl/ca-debian" ]; then
 fi
 export WGET="wget $WGET_OPTS"
 
+# Run the make_image step in parallel? Specify the number of calls to
+# use in parallel here if desired. Don't go too high - this *will*
+# thrash your IO!
+#export PARALLEL_MAKE_IMAGE=4
+
 # Set this to force the Release file(s) to say "stable". Used in first
 # Etch builds to allow us to build before the archive updated
 #EARLY_BUILD_HACK=1
