@@ -19,7 +19,7 @@ my ($basedir, $mirror, $tdir, $codename, $archlist, $mkisofs, $maxcds,
 my $mkisofs_base_opts = "";
 my $mkisofs_opts = "";
 my $mkisofs_dirs = "";
-my (@arches, @arches_nosrc, @overflowlist, @pkgs_added);
+my (@arches, @arches_nosrc, @overflowlist, @pkgs_added, @nonfree_components);
 my (@exclude_packages, @unexclude_packages, @excluded_package_list);
 my %firmware_package;
 my $current_checksum_type = "";
@@ -88,6 +88,7 @@ if ($maxisos < $maxcds) {
 }
 
 $extranonfree = read_env('EXTRANONFREE', 0);
+@nonfree_components = split /\ /, read_env('NONFREE_COMPONENTS', 'non-free');
 $nonfree = read_env('NONFREE', 0);
 $contrib = read_env('CONTRIB', 0);
 $use_local = read_env('LOCAL', 0);
@@ -172,6 +173,7 @@ if ($archlist =~ /m68k/ || $archlist =~ /powerpc/) {
 
 print "Starting to lay out packages into images:\n";
 
+# Read in the list of packages that we're expecting to include
 if (-e "$bdir/firmware-packages") {
     open(FWLIST, "$bdir/firmware-packages") or die "Unable to read firmware-packages file!\n";
     while (defined (my $pkg = <FWLIST>)) {
@@ -264,7 +266,7 @@ while (defined (my $pkg = <INLIST>)) {
     if (should_exclude_package($pkg)) {
         push(@excluded_package_list, $pkg);
     } elsif (should_start_extra_nonfree($pkg)) {
-        print LOG "Starting on extra non-free CDs\n";
+        print LOG "Starting on extra non-free image(s)\n";
         finish_disc($cddir, "");
         # And reset, to start the next disc
         $size = 0;
@@ -416,7 +418,9 @@ sub load_all_descriptions {
 	    load_descriptions("contrib", $use_backports);
 	}
 	if ($nonfree || $extranonfree) {
-	    load_descriptions("non-free", $use_backports);
+	    foreach my $component (@nonfree_components) {
+		load_descriptions($component, $use_backports);
+	    }
 	}
 	if ($use_local) {
 	    load_descriptions("local", $use_backports);
@@ -489,12 +493,15 @@ sub should_start_extra_nonfree {
     my $pkg = shift;
     my ($arch, $component, $pkgname, $pkgsize) = split /:/, $pkg;
 
-	if ( ($component eq "non-free") && $extranonfree) {
+    if ($extranonfree) {
+	foreach my $nf_comp (@nonfree_components) {
+	    if ($component eq $nf_comp) {
 		$extranonfree = 0; # Flag that we don't need to start new next time!
 		return 1;
+	    }
 	}
-	
-	return 0;
+    }
+    return 0;
 }
 
 sub should_exclude_package {
@@ -655,13 +662,13 @@ sub recompress {
 	# Packages and Sources files; workaround for bug #402482
 	if ($filename =~ m/\/.*\/(Packages|Sources)$/o) {
 		system("rm -f $_.gz");
-		system("gzip -9c < $_ >$_.gz");
+		system("gzip --no-name -9c < $_ >$_.gz");
 	}
 	# Translation files need to be compressed in .gz format on CD?
 	if ($filename =~ m/\/.*\/i18n\/(Translation.*)$/o &&
 		! ($filename =~ m/\/.*\/i18n\/(Translation.*gz)$/o)) {
 		system("rm -f $_.gz");
-		system("gzip -9c < $_ >$_.gz");
+		system("gzip --no-name -9c < $_ >$_.gz");
 		system("rm -f $_");
 	}
 }	
@@ -909,7 +916,7 @@ sub finish_disc {
 	print "  Finishing off md5sum.txt\n";
 	# Just md5 the bits we won't have seen already
 	open(MD5LIST, ">>md5sum.txt") or die "Failed to open md5sum.txt file: $!\n";
-	find (\&md5_files_for_md5sum, ("./.disk", "./dists"));
+	find (\&md5_files_for_md5sum, ("./.disk", "./dists", "./firmware/dep11"));
 	close(MD5LIST);
 
 	# And sort; it should make things faster for people checking
@@ -966,6 +973,8 @@ sub Packages_dir {
         $dist = "contrib";
     } elsif ($file =~ /\/non-free\//) {
         $dist = "non-free";
+    } elsif ($file =~ /\/non-free-firmware\//) {
+        $dist = "non-free-firmware";
     } else {
         $dist = "local";
     }	
@@ -1063,8 +1072,8 @@ sub add_trans_desc_entry {
 
     m/^Package: (\S+)/m and $p = $1;
     m/^Section: (\S+)/m and $section = $1;
-
     m/^Filename: (\S+)/mi and $file = $1;
+
     $idir = Packages_dir($dir, $file, $section, $in_backports) . "/i18n";
 
     if (! -d $idir) {
@@ -1164,6 +1173,88 @@ sub add_md5_entry {
     return $blocks_added;
 }
 
+# Add sym-links and pattern files for firmware packages
+sub add_firmware_stuff {
+    my $dir = shift;
+    my $arch = shift;
+    my $in_backports = shift;
+    local $_ = shift;
+    my ($p, $file, $section, $dist, $dep11_dir);
+    my $blocks_added = 0;
+    my @args = ("$basedir/tools/generate_firmware_patterns",
+		"--output-dir", "$dir/firmware/dep11");
+
+    m/^Package: (\S+)/m and $p = $1;
+    m/^Section: (\S+)/m and $section = $1;
+    m/^Filename: (\S+)/mi and $file = $1;
+
+    my $base_file = basename($file);
+
+    if ($file =~ /\/main\//) {
+        $dist = "main";
+    } elsif ($file =~ /\/contrib\//) {
+        $dist = "contrib";
+    } elsif ($file =~ /\/non-free\//) {
+        $dist = "non-free";
+    } elsif ($file =~ /\/non-free-firmware\//) {
+        $dist = "non-free-firmware";
+    } else {
+        $dist = "local";
+    }
+
+    $dep11_dir = "$mirror/dists/$codename/$dist/dep11";
+    if ($in_backports) {
+	$dep11_dir = "$mirror/dists/$codename-backports/$dist/dep11";
+    }
+
+    msg_ap(0, "Symlink fw package $p into /firmware\n");
+    symlink("../$file", "$dir/firmware/$base_file");
+    msg_ap(0, "Symlink ../$file $dir/firmware/.\n");
+    if (! -d "$dir/firmware") {
+	mkdir "$dir/firmware" or die "mkdir $dir/firmware failed $!\n";
+	mkdir "$dir/firmware/dep11" or die "mkdir $dir/firmware/dep11 failed $!\n";
+	$blocks_added += 2;
+    }
+
+    # Cope with maybe having the patterns file already
+    # (e.g. multi-arch), in which case we'll replace it here
+    if (-f "$dir/firmware/dep11/$p.patterns") {
+	$blocks_added -= get_file_blocks("$dir/firmware/dep11/$p.patterns");
+    }
+
+    msg_ap(0, "(Maybe) generate fw pattern file $dir/firmware/dep11/$p.patterns\n");
+    push(@args, "--package", "$p");
+    push(@args, "$dep11_dir/Components-$arch.yml.gz");
+    system(@args) == 0 or die "generate_firmware_patterns failed: $?";
+    if (-f "$dir/firmware/dep11/$p.patterns") {
+	$blocks_added += get_file_blocks("$dir/firmware/dep11/$p.patterns");
+    }
+
+    # Find the current size of the firmware Contents file
+    my $contents_blocks_old = 0;
+    my $cont_file = "$dir/firmware/Contents-firmware";
+    if (-f "$cont_file") {
+	$contents_blocks_old = get_file_blocks("$cont_file");
+    }
+    # Add new contents stuff, and count it
+    open(OFILE, ">> $cont_file");
+    open(DPKGC, "dpkg --contents $dir/firmware/$base_file |")
+	or die "Can't find contents of $dir/firmware/$base_file: $!";
+    while (defined(my $line = <DPKGC>)) {
+	chomp $line;
+	if ($line =~ m,^[-|l]\S+\s+\S+\s+\d+\s+\S+\s+\S+\s+./(\S+/firmware/\S+)$,) {
+	    printf OFILE "%-55s %s\n", $1, $base_file;
+	}
+    }
+    close DPKGC;
+    close OFILE;
+
+    my $contents_blocks_new = get_file_blocks($cont_file);
+    $blocks_added += $contents_blocks_new - $contents_blocks_old;
+
+    return $blocks_added;
+}
+
 # Roll back the results of add_Packages_entry()
 sub remove_Packages_entry {
     my $dir = shift;
@@ -1363,6 +1454,52 @@ sub remove_md5_entry {
     return $blocks_removed;
 }
 
+sub remove_firmware_stuff {
+    my $dir = shift;
+    my $arch = shift;
+    my $in_backports = shift;
+    my ($p, $file);
+    local $_ = shift;
+    my $blocks_removed = 0;
+
+    m/^Package: (\S+)/mi and $p = $1;
+    m/^Filename: (\S+)/mi and $file = $1;
+
+    my $base_file = basename($file);
+
+    msg_ap(0, "Remove symlink for fw package $p in /firmware\n");
+	unlink("$dir/firmware/$base_file");
+
+    if (-f "$dir/firmware/dep11/$p.patterns") {
+	$blocks_removed += get_file_blocks("$dir/firmware/dep11/$p.patterns");
+	msg_ap(0, "Remove $dir/firmware/dep11/$p.patterns\n");
+	unlink("$dir/firmware/dep11/$p.patterns");
+    }
+
+    # Find the current size of the firmware Contents file, and grep
+    # out from the current data
+    my $contents_blocks_old = 0;
+    my $cont_file = "$dir/firmware/Contents-firmware";
+    open(OFILE, "> $cont_file.1");
+    if (-f $cont_file) {
+	$contents_blocks_old = get_file_blocks($cont_file);
+	open(IFILE, "< $cont_file");
+	while (defined(my $line = <IFILE>)) {
+	    chomp $line;
+	    if ($line !~ /\b$base_file$/) {
+		print OFILE "$line\n";
+	    }
+	}
+	close IFILE;
+    }
+    close OFILE;
+    rename "$cont_file.1", "$cont_file";
+    my $contents_blocks_new = get_file_blocks($cont_file);
+    $blocks_removed += $contents_blocks_new - $contents_blocks_old;
+
+    return $blocks_removed;
+}
+
 sub get_file_blocks {
     my $realfile = shift;
     my $st;
@@ -1431,6 +1568,10 @@ sub add_packages {
             if (!($arch eq "source")) {
                 $total_blocks -= remove_trans_desc_entry($dir, $arch, $in_backports, $package_info);
             }
+	    
+	    if ($firmware_package{$pkgname}) {
+		$total_blocks -= remove_firmware_stuff($dir, $arch, $in_backports, $package_info);
+	    }
         
             foreach my $file (@files) {
                 my $missing = 0;
@@ -1450,6 +1591,9 @@ sub add_packages {
                 unlink ("$dir/$file") || msg_ap(0, "Couldn't delete file $dir/$file\n");
                 msg_ap(0, "  Rollback: removed $dir/$file\n");
             }
+	    # Try to remove the directory; will silently fail if there
+	    # are still files there, which is OK.
+	    rmdir ($dir);
         } else {
             $total_blocks += add_Packages_entry($dir, $arch, $in_backports, $package_info);
             $total_blocks += add_md5_entry($dir, $arch, $in_backports, $package_info);
@@ -1479,12 +1623,7 @@ sub add_packages {
                     $total_blocks += good_link ($realfile, "$dir/$file");
                     msg_ap(0, "  Linked $dir/$file\n");
                     if ($firmware_package{$pkgname}) {
-                        msg_ap(0, "Symlink fw package $pkgname into /firmware\n");
-                        if (! -d "$dir/firmware") {
-                            mkdir "$dir/firmware" or die "symlink failed $!\n";
-                        }
-                        symlink("../$file", "$dir/firmware/" . basename($file));
-                        msg_ap(0, "Symlink ../$file $dir/firmware/.\n");
+			$total_blocks += add_firmware_stuff($dir, $arch, $in_backports, $package_info);
                     }
                 } else {
                     msg_ap(0, "  $dir/$file already linked in\n");
